"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var webpack_1 = __importDefault(require("webpack"));
var checkRequiredFiles_1 = __importDefault(require("react-dev-utils/checkRequiredFiles"));
var webpack_merge_1 = __importDefault(require("webpack-merge"));
var webpack_config_1 = __importDefault(require("../webpack.config"));
var utils_1 = require("../utils");
var jsdom_1 = __importDefault(require("jsdom"));
var fs_1 = __importDefault(require("fs"));
var paths_1 = __importDefault(require("../config/paths"));
var JSDOM = jsdom_1.default.JSDOM;
var prepareCacheFolder = function () {
    var destPath = paths_1.default.appCacheStaticAbs;
    if (!fs_1.default.existsSync(destPath)) {
        fs_1.default.mkdirSync(destPath, {
            recursive: true,
        });
    }
};
/**
 *
 * @param filename
 * @param fileContent
 * @returns absolute cache file path
 */
var saveFileToCacheFolder = function (filename, fileContent) {
    var cacheFolderPath = paths_1.default.appCacheAbs;
    var filePath = path_1.default.resolve(cacheFolderPath, filename);
    fs_1.default.writeFileSync(filePath, fileContent, {
        encoding: 'utf8',
    });
    return filePath;
};
/**
 * @param filePath absolute file path
 */
// TODO: add file hash
var copyFileToCacheFolder = function (filePath, toStatic) {
    if (toStatic === void 0) { toStatic = false; }
    var folderPath = toStatic ? paths_1.default.appCacheStaticAbs : paths_1.default.appCacheAbs;
    var filename = utils_1.getFilename(filePath);
    var destFilePath = path_1.default.resolve(folderPath, filename);
    fs_1.default.copyFileSync(filePath, destFilePath);
    return destFilePath;
};
/**
 * @param htmlFilePath absolute file path
 */
// TODO: 支持 watch html 文件变化
var prepareHtmlFile = function (filePath) {
    if (filePath === void 0) { filePath = paths_1.default.defaultHtmlFileAbs; }
    var filename = utils_1.getFilename(filePath);
    var html = fs_1.default.readFileSync(filePath, {
        encoding: 'utf8',
    });
    var dom = new JSDOM(html);
    var entry = [];
    // 如果没有 filePath 说明用的是默认 template，而默认 template 里面没有 script
    if (filePath) {
        prepareCacheFolder();
        var resolveFilePathInHtml_1 = function (relPath) {
            var htmlFolderPath = path_1.default.resolve(path_1.default.relative(utils_1.resolvePathInProject('.'), filePath), '../');
            return utils_1.resolvePathInProject(path_1.default.resolve(htmlFolderPath, relPath));
        };
        var handleCopy = function (attrName) { return function (node) {
            var src = node.getAttribute(attrName);
            if (src) {
                var filePath_1 = resolveFilePathInHtml_1(src);
                var newFilePath = copyFileToCacheFolder(filePath_1, true);
                var newFilename = utils_1.getFilename(newFilePath);
                // TODO: base path
                var newSrc = "/static/" + newFilename;
                node.setAttribute(attrName, newSrc);
            }
        }; };
        // link, img 资源拷贝
        dom.window.document.querySelectorAll('body img').forEach(handleCopy('src'));
        dom.window.document.querySelectorAll('link').forEach(handleCopy('href'));
        // 获取 entry
        dom.window.document.querySelectorAll('body script').forEach(function (node) {
            var src = node.getAttribute('src');
            if (src) {
                var scriptSrc = resolveFilePathInHtml_1(src);
                entry.push(scriptSrc);
                dom.window.document.body.removeChild(node);
            }
        });
    }
    var html2 = dom.serialize();
    var generatedHtmlAbsPath = saveFileToCacheFolder(filename, html2);
    return {
        htmlPath: generatedHtmlAbsPath,
        entry: entry.length ? entry : null,
    };
};
exports.initCompiler = function (options) {
    var configFilePath = options.configFilePath, entryFilePath = options.entryFilePath, rest = __rest(options, ["configFilePath", "entryFilePath"]);
    if (!path_1.default.isAbsolute(entryFilePath)) {
        entryFilePath = utils_1.resolvePathInProject(entryFilePath);
    }
    var htmlFilePath;
    var webpackEntry;
    if (entryFilePath.endsWith('.html')) {
        var _a = prepareHtmlFile(entryFilePath), htmlPath = _a.htmlPath, entry = _a.entry;
        if (!entry) {
            utils_1.print.warn("no entry was found in " + entryFilePath);
        }
        htmlFilePath = htmlPath;
        webpackEntry = entry;
    }
    else {
        var htmlPath = prepareHtmlFile(entryFilePath).htmlPath;
        htmlFilePath = htmlPath;
        webpackEntry = entryFilePath;
    }
    // 用户指定的 webpack 配置文件
    var webpackConfig = {};
    if (configFilePath) {
        if (!checkRequiredFiles_1.default([configFilePath])) {
            process.exit(1);
        }
        webpackConfig = require(configFilePath);
        utils_1.print.log(configFilePath + " is being used");
    }
    var finalWebpackConfig = webpack_merge_1.default(webpack_config_1.default(__assign({}, rest, { htmlFilePath: htmlFilePath, entry: webpackEntry })), webpackConfig);
    return {
        compiler: webpack_1.default(finalWebpackConfig),
        webpackConfig: finalWebpackConfig,
    };
};
//# sourceMappingURL=webpack.js.map